// $Id: ListenTimeData.java,v 1.1 2007/05/23 21:52:50 osoysal Exp $

/*									tab:4
 * "Copyright (c) 2000-2003 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Copyright (c) 2002-2003 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE     
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA, 
 * 94704.  Attention:  Intel License Inquiry.
 */

/** modified from original Listen app. 
This one is specific to pollcast
Much better package handling could have been done
*/
import java.io.*;
import java.util.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import net.tinyos.message.*;
import javax.swing.*;
import java.awt.*;
class DisplayFrame extends JFrame
{
	Vector<Integer> local;
	int recvCount;
	DisplayFrame()
	{
		local=new Vector<Integer>();
		this.setBounds(0,0,ListenTimeData.WINDOW_SIZE,600);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}
	
	public void paint(Graphics g)
	{
		synchronized(this)
		{
			setBackground(Color.black);
			g.clearRect(0,0,ListenTimeData.WINDOW_SIZE,600);
			int size=ListenTimeData.arr.size();
			int redCount=0,greenCount=0,collusionCount=0;
			for (int i=0;i<local.size();i++)
			{
				int y1=local.get(i);
				if (y1>0)
				{				
					collusionCount++;
					if (y1<101)
					{
						redCount++;
						g.setColor(Color.red);
					}
					else if (y1<201)
					{
						greenCount++;
						y1/=2;
						g.setColor(Color.green);
					}
					else
					{
						redCount++;
						greenCount++;
						y1/=3;
						g.setColor(Color.yellow);
					}
				}
				g.drawLine(i, 600, i,600-y1*2);
			}
			g.setColor(Color.white);
			g.drawString("Recieved Packet Count: "+recvCount,50,50);
			g.setColor(Color.red);
			g.drawString("CRC Based Collusion Detections: "+redCount,50,70);
			g.setColor(Color.green);
			g.drawString("CCA Based Collusion Detections: "+greenCount,50,90);
			g.setColor(Color.white);
			g.drawString("Total Collusion Detections: "+collusionCount,50,110);
		}
	}
	
	void updateData(Vector<Integer> arr,int recvCount)
	{
		synchronized(this)
		{
			local.clear();
			int start=0;
			if (arr.size()>ListenTimeData.WINDOW_SIZE)
				start=arr.size()-ListenTimeData.WINDOW_SIZE;
			for (int i=start;i<arr.size();i++)
				local.add(arr.get(i));
			this.recvCount=recvCount;
		}
	}
}
enum PacketType
{
	UNKNOWN,
	RSSI,
	CCA,
	STATE_REPORT,
	RECIEVE_COUNT,
	RSCD
};

public class ListenTimeData
{
	static Vector<Integer> arr;
	static DisplayFrame df;
	static int WINDOW_SIZE;
	static int recvPackets;
	public static void parsePacket(byte [] packet)
	{
		PacketType pt;
		switch(packet[12])
		{
			case 1:
				pt=PacketType.RSSI;
				break;
			case 2:
				pt=PacketType.CCA;
				break;
			case 3:
				pt=PacketType.STATE_REPORT;
				break;
			case 4:
				pt=PacketType.RECIEVE_COUNT;
				break;
			case 5:
				pt=PacketType.RSCD;
				break;
			default:
				pt=PacketType.UNKNOWN;
				break;
		}
		if (pt==PacketType.CCA || pt==PacketType.RSSI || pt==PacketType.STATE_REPORT || pt==PacketType.RSCD)
		{
			for (int i=0;i<packet[14];i++)
			{
				int t=(int)packet[15+i];
				if (pt==PacketType.RSSI)
					t+=100; // some baseline for RSSI
				else if (pt==PacketType.CCA || pt==PacketType.RSCD)
					t=t*100; // scale CCA values and collusion detection values
				else if (pt==PacketType.STATE_REPORT)
				{
					if (t<0)
						t+=256;// fix sign bit
				}
				arr.add(t);
//			System.out.print(128+packet[14+i]+" ");
			}
			int t=packet[15+17];
			if (t<0)
				t=t+256;
			int t2=packet[15+16];
			if (t2<0)
				t2=t2+256;
			recvPackets=t+t2*256;
			if (pt==PacketType.STATE_REPORT)
				printStatePackage(packet);
		}
		else if (pt==PacketType.RECIEVE_COUNT)
		{
			int t=packet[15+17];
			if (t<0)
				t=t+256;
			int t2=packet[15+16];
			if (t2<0)
				t2=t2+256;
			recvPackets=t+t2*256;
		}
	}
	
	static void printStatePackage(byte [] packet)
	{
		for (int i=0;i<packet[14];i++)
		{
			int t=packet[15+i];
			if (t<0)
				t+=256;
			System.out.print(t+" ");
		}
		//System.out.println();
	}
    public static void main(String args[]) throws IOException 
	{
		WINDOW_SIZE=900;
		recvPackets=0;
		arr=new Vector<Integer>();
		df=new DisplayFrame();
		df.setVisible(true);
	if (args.length > 0) {
	    System.err.println("usage: java ListenOnur");
	    System.exit(2);
	}
	
	PacketSource reader = BuildSource.makePacketSource();
	if (reader == null) {
	    System.err.println("Invalid packet source (check your MOTECOM environment variable)");
	    System.exit(2);
	}
	//tstates=new int[255][255];

	try {
	  reader.open(PrintStreamMessenger.err);
	  for (int j=0;;j++) {
	    byte[] packet = reader.readPacket();
//		System.out.printf("%4d:",j);
	    Dump.printPacket(System.out, packet);
	    System.out.println();
//		parsePacket(packet);
	    System.out.flush();
	/*	if (packet[12]==4 || packet[14]!=16) // ugly hack for that 
		{
			df.updateData(arr,recvPackets);
			df.repaint();
		}*/
	  }
	}
	catch (IOException e) {
	    System.err.println("Error on " + reader.getName() + ": " + e);
	}
    }
}

